from cryptography.fernet import Fernet
import cryptography


def decrypt(filename, key):
    """
    Given a filename (str) and key (bytes), it decrypts the file and write it
    """
    f = Fernet(key)
    with open(filename, "rb") as file:
        # read the encrypted data
        encrypted_data = file.read()
    # decrypt data
    try:
        decrypted_data = f.decrypt(encrypted_data)
    except cryptography.fernet.InvalidToken:
        return True
    # write the original file
    with open(filename, "wb") as file:
        file.write(decrypted_data)



def check_for_salt(os, window):
    if os.path.exists('./../salt.salt'):
        window['EnterMasterPasswordText'].Update(visible=True)
        window['EnterMasterPasswordInput'].Update(visible=True)
        window['EnterExistingMasterPasswordButton'].Update(visible=True)
    else:
        window['CreateMasterPasswordText'].Update(visible=True)
        window['CreateMasterPasswordInput'].Update(visible=True)
        window['CreateMasterPasswordButton'].Update(visible=True)


import os.path
import PySimpleGUI as sg

from generate_key import *
from check_for_salt import *
from show_password_button_event import *
from enter_existing_master_password_button import *
from create_master_password_button_event import *
from enter_password_button_event import *
from build_layout import *

global key

sg.theme('DarkAmber')

layout = build_layout(sg)

window = sg.Window('Password Manager', layout, finalize=True)

check_for_salt(os, window)

while True:
    global masterPassword
    event, values = window.read()

    if event == sg.WIN_CLOSED or event == 'Cancel':  # if user closes window or clicks cancel
        break

    if event == 'showPasswordsButton':
        show_password_button_event(key, window, values)

    if event == 'EnterExistingMasterPasswordButton':
        key = generate_key(values['EnterMasterPasswordInput'], load_existing_salt=True)
        enter_existing_master_password_button(key, values, window)

    if event == 'CreateMasterPasswordButton':
        key = generate_key(values['CreateMasterPasswordInput'])
        create_master_password_button_event(key, window, os)

    if event == 'EnterPasswordButton':
        enter_password_button_event(key, window, values)


def show_form(window):
    window['UsernameFormText'].Update(visible=True)
    window['WebsiteFormName'].Update(visible=True)
    window['PasswordFormName'].Update(visible=True)
    window['EnterPasswordButton'].Update(visible=True)
    window['CancelButton'].Update(visible=True)
    window['UsernameFormInput'].Update(visible=True)
    window['WebsiteFormInput'].Update(visible=True)
    window['PasswordFormInput'].Update(visible=True)


from cryptography.hazmat.primitives.kdf.scrypt import Scrypt


def derive_key(salt, password):
    """Derive the key from the `password` using the passed `salt`"""
    kdf = Scrypt(salt=salt, length=32, n=2**14, r=8, p=1)
    return kdf.derive(password.encode())



from decrypt import *
import subprocess as sp
from encrypt import *


def show_password_button_event(key, window, values):
    with open('./../passwordFile.txt') as f:
        err = decrypt('./../passwordFile.txt', key)
        if err:
            window['InvalidPasswordText'].Update(visible=True)
            values['EnterMasterPasswordInput'] = ''
        else:
            program_name = "notepad.exe"
            file_name = "./../passwordFile.txt"
            process = sp.Popen([program_name, file_name])
            exitcode = process.wait()
            if exitcode == 0:
                encrypt('./../passwordFile.txt', key)


def build_layout(sg):
    return [[sg.pin(sg.Text('Enter Username here (optional)', key='UsernameFormText', visible=False)),
           sg.pin(sg.InputText('', key='UsernameFormInput', visible=False, do_not_clear=False))],
          [sg.pin(sg.Text('Enter website here (optional)', key='WebsiteFormName', visible=False)),
           sg.pin(sg.InputText('', key='WebsiteFormInput', visible=False, do_not_clear=False))],
          [sg.pin(sg.Text('Enter length of password here', key='PasswordFormName', visible=False)),
           sg.pin(sg.InputText('', key='PasswordFormInput', visible=False, do_not_clear=False))],
          [sg.pin(sg.Button('Enter', key='EnterPasswordButton', visible=False)),
           sg.pin(sg.Button('Cancel', key='CancelButton', visible=False))],
          [sg.pin(sg.Button('Show Passwords', key='showPasswordsButton', visible=False))],
          [sg.pin(sg.Text('Enter master password', key='EnterMasterPasswordText', visible=False)),
           sg.pin(sg.InputText('', key='EnterMasterPasswordInput', visible=False, do_not_clear=False))],
          [sg.pin(sg.Text('Create master password', key='CreateMasterPasswordText', visible=False)),
           sg.pin(sg.InputText('', key='CreateMasterPasswordInput', visible=False, do_not_clear=False))],
          [sg.pin(sg.Button('Enter', key='EnterExistingMasterPasswordButton', visible=False))],
          [sg.pin(sg.Button('Enter', key='CreateMasterPasswordButton', visible=False))],
          [sg.pin(sg.Text('Invalid Password', key='InvalidPasswordText', visible=False))]]


from decrypt import *
from show_form import *
from encrypt import *


def enter_existing_master_password_button(key, values, window):
    err = decrypt('../passwordFile.txt', key)

    if err:
        window['InvalidPasswordText'].Update(visible=True)
        values['EnterMasterPasswordInput'] = ''
    else:
        encrypt('../passwordFile.txt', key)

        show_form(window)

        window['EnterMasterPasswordText'].Update(visible=False)
        window['EnterMasterPasswordInput'].Update(visible=False)
        window['EnterExistingMasterPasswordButton'].Update(visible=False)
        window['showPasswordsButton'].Update(visible=True)

        window['InvalidPasswordText'].Update(visible=False)


def insert_password(usernameforminput,
                    websiteforminput,
                    password_list,
                    f):
    if len(usernameforminput.strip()) > 0:
        f.write('Username: ' + usernameforminput + ' ')
    else:
        f.write('Username: N/A ')
    if len(websiteforminput.strip()) > 0:
        f.write('Website: ' + websiteforminput + ' ')
    else:
        f.write('Website:  N/A ')

    f.write('Password: ' + ''.join(password_list))
    f.write('\n')


from encrypt import *
from show_form import *


def create_master_password_button_event(key, window, os):
    if not os.path.exists('../passwordFile.txt'):
        open('../passwordFile.txt', 'w')

    encrypt('../passwordFile.txt', key)

    show_form(window)

    window['CreateMasterPasswordText'].Update(visible=False)
    window['CreateMasterPasswordInput'].Update(visible=False)
    window['CreateMasterPasswordButton'].Update(visible=False)
    window['showPasswordsButton'].Update(visible=True)


import random
from decrypt import *
from insert_password import *
from encrypt import *


def enter_password_button_event(key, window, values):
    password_list = []
    err = decrypt('../passwordFile.txt', key)
    if err:
        window['InvalidPasswordText'].Update(visible=True)
        values['EnterMasterPasswordInput'] = ''
    else:
        print(password_list)
        for x in range(int(values['PasswordFormInput'])):
            temp_list = []
            uppercase_letter = chr(random.randint(65, 90))
            temp_list.append(uppercase_letter)

            lowercase_letter = chr(random.randint(97, 122))
            temp_list.append(lowercase_letter)

            digit = chr(random.randint(48, 57))
            temp_list.append(digit)

            hashtag = chr(35)
            temp_list.append(hashtag)

            exclamation = chr(33)
            temp_list.append(exclamation)

            password_list.append(random.choice(temp_list))
            pass
        else:
            with open('../passwordFile.txt', 'a') as f:
                insert_password(values['UsernameFormInput'],
                                values['WebsiteFormInput'],
                                password_list,
                                f)
            encrypt('../passwordFile.txt', key)
            pass



from setuptools import setup, find_packages

from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='password_manager_py',
    version='1.1.0',
    author="SMc",
    author_email="jackagusjill21@gmail.com",
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords='password manager',
    install_requires=[
        'PySimpleGUI',
        'cryptography'
    ]
)


from cryptography.fernet import Fernet


def encrypt(filename, key):
    """
    Given a filename (str) and key (bytes), it encrypts the file and write it
    """
    f = Fernet(key)
    with open(filename, "rb") as file:
        # read all file data
        file_data = file.read()
    # encrypt data
    encrypted_data = f.encrypt(file_data)
    # write the encrypted file
    with open(filename, "wb") as file:
        file.write(encrypted_data)



import base64

from load_salt import *
from generate_salt import *
from derive_key import *


def generate_key(password, salt_size=16, load_existing_salt=False, save_salt=True):
    """
    Generates a key from a `password` and the salt.
    If `load_existing_salt` is True, it'll load the salt from a file
    in the current directory called "salt.salt".
    If `save_salt` is True, then it will generate a new salt
    and save it to "salt.salt"
    """
    if load_existing_salt:
        # load existing salt
        salt = load_salt()
    elif save_salt:
        # generate new salt and save it
        salt = generate_salt(salt_size)
        with open("../salt.salt", "wb") as salt_file:
            salt_file.write(salt)
    # generate the key from the salt and the password
    derived_key = derive_key(salt, password)
    # encode it using Base 64 and return it
    return base64.urlsafe_b64encode(derived_key)